/* THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
 * ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * Copyright (C) 1994  Microsoft Corporation.  All Rights Reserved.
 *
 *  MODULE:   bunchq.c
 *
 *  PURPOSE:
 *	  contains functions relating to queues of bunches
 *
 *  FUNCTIONS:
 *	  GrabFirstBunch(LPBUNCHQUEUE lpbq)
 *	  GrabBunch(LPBUNCHQUEUE lpbq, LONG lBunchId)
 *	  GrabEmptyBunch(LPBUNCHQUEUE lpbq)
 *	  QueueBunch(LPBUNCHQUEUE lpbq, LONG lSlot)
 *	  QueueBunchAtHead(LPBUNCHQUEUE lpbq, LONG lSlot)
 *	  ReserveSlot(LPBUNCHQUEUE lpbq)
 *	  UnreserveSlot(LPBUNCHQUEUE lpbq)
 *	  FindBunch(LPBUNCHQUEUE lpbq, LONG lBunchId)
 *	  ClearDownBunch(LPBUNCHQUEUE lpbq, LONG lSlot)
 *
 *  COMMENTS:
 *    Each queue is a linked list of bunches from an array of bunch structures.
 *    In the sample, all the queues share the same bunch array pool for
 *    efficient transfer of bunches from one queue to another.  The structures
 *    in the pool are referred to as 'slots', and a 'slot number' is an index
 *    into the array of bunch structures.
 *
 *    Each queue has its own critical section for thread synchronisation, and
 *    the pool has its own critical section as well.
 *
 *    The 'Queue' functions are used to place a bunch on a queue.  When a bunch
 *    is on a queue, its status member is set to DCOMP_QQUEUED.
 *
 *    The 'Grab' functions are used to get a bunch  for some activity (either
 *    an empty one or one from a queue).  The status is set to DCOMP_QBUSY when
 *    the bunch structure is in use, either by an RPC call (transferring the
 *    bunch data to or from the client) or by a processing thread.  When a
 *    bunch's status is DCOMP_QBUSY, it is not on any queue.
 *
 *    The 'Queue' and 'Grab' functions do not manipulate the nQCnt member of
 *    the queue structure, which are used when testing whether a queue is full.
 *    The ReserveSlot and UnreserveSlot functions manipulate this value.
 *
 *    The nQCnt value is not strictly the number of bunches in a queue, but the
 *    number of slots reserved in the queue.  Although not queued, a busy bunch
 *    always has a slot reserved on at least one queue.  This is because it may
 *    be placed on the queue afterwards.  For example, a new bunch being copied
 *    during an RPC call will end up on an input queue.  Similarly, a bunch
 *    which has been grabbed for processing may be placed on an output queue
 *    afterwards, or may have to be returned to the input queue to be re-tried
 *    if the operation fails.
 */

#include	"dcomp.h"
#include	"dcomp_s.h"


//
//  FUNCTION: GrabFirstBunch
//
//  PURPOSE:
//    take first bunch off the head of a queue and mark it busy
//
//  PARAMETERS:
//    lpbq - pointer to the BUNCHQUEUE structure
//
//  RETURN VALUE:
//    Slot number of the returned bunch in the queue's pool.
//    DCOMP_QEND if queue is empty.
//    
//
//  COMMENTS:
//

LONG		GrabFirstBunch(LPBUNCHQUEUE lpbq)
{
	LONG		lSlot;

	EnterCriticalSection(&lpbq->csQ);

	if ((lSlot = lpbq->nQHead) != DCOMP_QEND)
	{
		lpbq->nQHead = lpbq->abnchPool[lSlot].nNext;
		lpbq->abnchPool[lSlot].nNext = DCOMP_QEND;
		lpbq->abnchPool[lSlot].nStat = DCOMP_QBUSY;
	}

	LeaveCriticalSection(&lpbq->csQ);
	return(lSlot);
}


//
//  FUNCTION: GrabBunch
//
//  PURPOSE:
//    remove bunch with a specified bunch ID from a queue and mark it busy
//
//  PARAMETERS:
//    lpbq     - pointer to the BUNCHQUEUE structure
//    lBunchId - the required bunch ID
//
//  RETURN VALUE:
//    Slot number of the returned bunch in the queue's pool.
//    DCOMP_QEND if specified bunch is not on the queue.
//
//  COMMENTS:
//

LONG		GrabBunch(LPBUNCHQUEUE lpbq, LONG lBunchId)
{
	LONG		lSlot;
	LONG		lPrev;

	EnterCriticalSection(&lpbq->csQ);

	if ((lSlot = lpbq->nQHead) == DCOMP_QEND)
	{
		LeaveCriticalSection(&lpbq->csQ);
		return(DCOMP_QEND);
	}

	while (lpbq->abnchPool[lSlot].lBunchId != lBunchId)
	{
		if (lpbq->abnchPool[lSlot].nNext == DCOMP_QEND)
		{
			LeaveCriticalSection(&lpbq->csQ);
			return(DCOMP_QEND);
		}

		lPrev = lSlot;
		lSlot = lpbq->abnchPool[lSlot].nNext;
	}

	if (lSlot == lpbq->nQHead)
	{
		lpbq->nQHead = lpbq->abnchPool[lSlot].nNext;
	}
	else
	{
		lpbq->abnchPool[lPrev].nNext = lpbq->abnchPool[lSlot].nNext;
	}

	lpbq->abnchPool[lSlot].nNext = DCOMP_QEND;
	lpbq->abnchPool[lSlot].nStat = DCOMP_QBUSY;

	LeaveCriticalSection(&lpbq->csQ);
	return(lSlot);
}


//
//  FUNCTION: GrabEmptyBunch
//
//  PURPOSE:
//    take an empty bunch from a queue's pool and mark it busy
//
//  PARAMETERS:
//    lpbq - pointer to the BUNCHQUEUE structure
//
//  RETURN VALUE:
//    Slot number of the returned bunch in the queue's pool.
//    DCOMP_QEND if there are no empty bunches in the pool.
//
//  COMMENTS:
//

LONG		GrabEmptyBunch(LPBUNCHQUEUE lpbq)
{
	LONG		lSlot;

	EnterCriticalSection(&lpbq->csQ);
	EnterCriticalSection(lpbq->lpcsPool);

	for (lSlot = 0L; lSlot < lpbq->nPoolLen; lSlot++)
	{
		if (lpbq->abnchPool[lSlot].nStat == DCOMP_QEMPTY)
		{
			lpbq->abnchPool[lSlot].nNext = DCOMP_QEND;
			lpbq->abnchPool[lSlot].nStat = DCOMP_QBUSY;
			LeaveCriticalSection(lpbq->lpcsPool);
			LeaveCriticalSection(&lpbq->csQ);
			return(lSlot);
		}
	}

	LeaveCriticalSection(lpbq->lpcsPool);
	LeaveCriticalSection(&lpbq->csQ);
	return(DCOMP_QEND);
}


//
//  FUNCTION: QueueBunch
//
//  PURPOSE:
//    add a bunch to the end of a queue
//
//  PARAMETERS:
//    lpbq  - pointer to the BUNCHQUEUE structure
//    lSlot - slot number of the bunch in the queue's pool
//
//  RETURN VALUE:
//    Zero on success, DCOMP_QERR on error.  (Currently no errors are defined.)
//
//  COMMENTS:
//    Assumes a slot has already been reserved on the queue for the bunch.
//

LONG		QueueBunch(LPBUNCHQUEUE lpbq, LONG lSlot)
{
	LONG		nEnd;

	EnterCriticalSection(&lpbq->csQ);

	if (lpbq->nQHead == DCOMP_QEND)
	{
		lpbq->nQHead = lSlot;
	}
	else
	{
		nEnd = lpbq->nQHead;
		while (lpbq->abnchPool[nEnd].nNext != DCOMP_QEND)
		{
			nEnd = lpbq->abnchPool[nEnd].nNext;
		}
		lpbq->abnchPool[nEnd].nNext = lSlot;
	}

	lpbq->abnchPool[lSlot].nNext = DCOMP_QEND;
	lpbq->abnchPool[lSlot].nStat = DCOMP_QQUEUED;

	LeaveCriticalSection(&lpbq->csQ);
	return(0);
}


//
//  FUNCTION: QueueBunchAtHead
//
//  PURPOSE:
//    add a bunch to the beginning of a queue
//
//  PARAMETERS:
//    lpbq  - pointer to the BUNCHQUEUE structure
//    lSlot - slot number of the bunch in the queue's pool
//
//  RETURN VALUE:
//    Zero on success, DCOMP_QERR on error.  (Currently no errors are defined.)
//
//  COMMENTS:
//    Used when re-queuing bunches which were grabbed for some operation which
//    then failed.
//    Assumes a slot has already been reserved on the queue for the bunch.
//

LONG		QueueBunchAtHead(LPBUNCHQUEUE lpbq, LONG lSlot)
{
	EnterCriticalSection(&lpbq->csQ);

	lpbq->abnchPool[lSlot].nNext = lpbq->nQHead;
	lpbq->nQHead = lSlot;
	lpbq->abnchPool[lSlot].nStat = DCOMP_QQUEUED;

	LeaveCriticalSection(&lpbq->csQ);
	return(0);
}


//
//  FUNCTION: ReserveSlot
//
//  PURPOSE:
//    reserve space on a queue for a slot
//
//  PARAMETERS:
//    lpbq  - pointer to the BUNCHQUEUE structure
//
//  RETURN VALUE:
//    Zero on success, DCOMP_QERR if the queue is already full
//
//  COMMENTS:
//

LONG		ReserveSlot(LPBUNCHQUEUE lpbq)
{
	EnterCriticalSection(&lpbq->csQ);
	if (lpbq->nQCnt < lpbq->nQLen)
	{
		lpbq->nQCnt++;
		LeaveCriticalSection(&lpbq->csQ);
		return(0);
	}
	else
	{
		lpbq->nQCnt = lpbq->nQLen;
		LeaveCriticalSection(&lpbq->csQ);
		return(DCOMP_QERR);
	}
}


//
//  FUNCTION: UnreserveSlot
//
//  PURPOSE:
//    release a slot's space on a queue
//
//  PARAMETERS:
//    lpbq  - pointer to the BUNCHQUEUE structure
//
//  RETURN VALUE:
//    Zero on success, DCOMP_QERR if the queue is already empty
//
//  COMMENTS:
//

LONG		UnreserveSlot(LPBUNCHQUEUE lpbq)
{
	EnterCriticalSection(&lpbq->csQ);
	if (lpbq->nQCnt > 0L)
	{
		lpbq->nQCnt--;
		LeaveCriticalSection(&lpbq->csQ);
		return(0);
	}
	else
	{
		lpbq->nQCnt = 0L;
		LeaveCriticalSection(&lpbq->csQ);
		return(DCOMP_QERR);
	}
}


//
//  FUNCTION: FindBunch
//
//  PURPOSE:
//    return the slot number of the bunch with a specified bunch ID in a queue's
//    pool
//
//  PARAMETERS:
//    lpbq     - pointer to the BUNCHQUEUE structure
//    lBunchId - the required bunch ID
//
//  RETURN VALUE:
//    Slot number of the bunch in the queue's pool.
//    DCOMP_QEND if specified bunch is not on the queue.
//
//  COMMENTS:
//

LONG		FindBunch(LPBUNCHQUEUE lpbq, LONG lBunchId)
{
	LONG		lSlot;

	EnterCriticalSection(&lpbq->csQ);

	if ((lSlot = lpbq->nQHead) == DCOMP_QEND)
	{
		LeaveCriticalSection(&lpbq->csQ);
		return(DCOMP_QEND);
	}

	while (lpbq->abnchPool[lSlot].lBunchId != lBunchId)
	{
		if (lpbq->abnchPool[lSlot].nNext == DCOMP_QEND)
		{
			LeaveCriticalSection(&lpbq->csQ);
			return(DCOMP_QEND);
		}

		lSlot = lpbq->abnchPool[lSlot].nNext;
	}

	LeaveCriticalSection(&lpbq->csQ);
	return(lSlot);
}


//
//  FUNCTION: ClearDownBunch
//
//  PURPOSE:
//    free buffers belonging to a bunch and mark the bunch empty
//
//  PARAMETERS:
//    lpbq  - pointer to the BUNCHQUEUE structure
//    lSlot - slot number of the bunch in the queue's pool
//
//  RETURN VALUE:
//
//  COMMENTS:
//

void		ClearDownBunch(LPBUNCHQUEUE lpbq, LONG lSlot)
{
	// assumes bunch is in the DCOMP_QBUSY state

	ClearDown(&lpbq->abnchPool[lSlot].lpbiCom);
	ClearDown(&lpbq->abnchPool[lSlot].lpbiUnc);
	ClearDown(&lpbq->abnchPool[lSlot].lpbiRec);
	ClearDown(&lpbq->abnchPool[lSlot].lplComIndex);
	ClearDown(&lpbq->abnchPool[lSlot].lpadwRetFlags);
	ClearDown(&lpbq->abnchPool[lSlot].lpbComData);
	ClearDown(&lpbq->abnchPool[lSlot].lpbUncData);

	EnterCriticalSection(&lpbq->csQ);
	lpbq->abnchPool[lSlot].nNext = DCOMP_QEND;
	lpbq->abnchPool[lSlot].nStat = DCOMP_QEMPTY;
	LeaveCriticalSection(&lpbq->csQ);
}
