/* THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
 * ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
 * PARTICULAR PURPOSE.
 *
 * Copyright (C) 1994  Microsoft Corporation.  All Rights Reserved.
 *
 *  MODULE:   dcomp_c.c
 *
 *  PURPOSE:
 *	  contains client-side code for the DCompDistributed Compression sample
 *
 *  FUNCTIONS:
 *    ScanCommandLine(int argc, TCHAR *argv[])
 *    FindAndContactServers(void)
 *    ContactNamedServers(void)
 *    _tmain(int argc, TCHAR *argv[])
 *
 *  COMMENTS:
 *    The client divides a source AVI file into 'bunches' which are sent to a
 *    number of server processes running on remote machines.  The servers
 *    recompress the video, and the recompressed bunches are returned to the
 *    client to be re-assembled in the destination file.
 *
 *    This file contains the client main procedure and command-line parser, and
 *    the functions responsible for making contact with servers on start-up.
 *
 *    The client application can be built to use either UNICODE or ANSI APIs,
 *    according to the project settings.
 */

#define		DEFINE_GLOBALS
#include	"dcomp.h"
#include	"dcomp_c.h"


//
//  FUNCTION: ScanCommandLine
//
//  PURPOSE: parse the command line
//
//  PARAMETERS:
//    argc - number of command line arguments
//    argv - array of command line arguments
//
//  RETURN VALUE:
//    non-zero for error
//
//  COMMENTS:
//    Usage: dcomp_c <infile> [<outfile>] [<outformat>] [<computer>...]
//
//    If <outfile> is not specified, dcomp_c tries to generate a unique filename
//    If <outformat> is not specified, it defaults to MSVC
//    If computer names are specified, dcomp_c looks for servers on the named
//    computers.  If no names are specified, dcomp_c looks for servers using the
//    RPC Locator service
//
//    Extra 'secret' features:
//
//     dcomp_c -f
//       just find available servers and then quit
//
//     dcomp_c -h [<computer>]
//       just ask the server on the named computer (local computer if
//       not specified) to checkpoint its local heap.
//
//     dcomp_c <infile> [<outfile>] [<outformat>] [-c <comp>...] [-d <decomp>...]
//		 -c specifies that the following computers should do the compression
//		 -d specifies that the following computers should do the decompression
//

LONG		ScanCommandLine(int argc, TCHAR *argv[])
{
    TCHAR		*s;
	LONG		lArg = 1;
    enum
    {
        INPUTFILE, OUTPUTFILE, OUTCOMPRESSION,
        									DASH, DECOMPSERVER, COMPSERVER, ERR
    } eScanState = INPUTFILE;


    // Strip path (if any) from command name

    s = _tcsrchr(argv[0], TEXT('\\'));
    _tcsncpy(gacBaseName, (s == NULL ? argv[0] : s + 1 ), 8);

    if ((s = _tcschr(gacBaseName, TEXT('.'))) == NULL)
    {
    	s = &gacBaseName[8];
    }

    *s = TEXT('\0');


    // Loop through the command line arguments.  eScanState keeps track of the
    // command syntax.

    while ((lArg < argc) && (eScanState != ERR))
    {
    	switch(eScanState)
        {
        	case INPUTFILE:
            	if (*(argv[lArg]) == TEXT('-'))
                {
                	argv[lArg]++;
                	eScanState = DASH;
                    break;
                }
                else
                {
                	gszInputFileName = argv[lArg++];
                	eScanState = OUTPUTFILE;
                }

                if (lArg >= argc)
                {
                	return(0);
                }
                break;

			case OUTPUTFILE:
            	if (*(argv[lArg]) == TEXT('-'))
                {
                	argv[lArg]++;
                	eScanState = DASH;
                    break;
                }

                gszOutputFileName = argv[lArg++];
	            eScanState = OUTCOMPRESSION;

                if (lArg >= argc)
                {
                	return(0);
                }
                break;

			case OUTCOMPRESSION:
            	if (*(argv[lArg]) == TEXT('-'))
                {
                	argv[lArg]++;
                	eScanState = DASH;
                    break;
                }

                gdwOutCompression = mmioFOURCC(
								                *(argv[lArg]    ),
								                *(argv[lArg] + 1),
								                *(argv[lArg] + 2),
								                *(argv[lArg] + 3)
											  );

	            eScanState = DECOMPSERVER;

                if (++lArg >= argc)
                {
                	return(0);
                }
                break;

            case DASH:
                switch(*(argv[lArg]))
                {
                    case TEXT('d'):
                    case TEXT('r'):		// Specifies recompression servers
                    					// - but not checked for consistency:
                    					// if compression servers are
                    					// specified, these recompression
                    					// servers will become decompression
                    					// servers.
		                eScanState = DECOMPSERVER;
                        if (*(++argv[lArg]) == TEXT('\0'))
                        {
                            lArg++;
                        }
                        break;

                    case TEXT('c'):
		                eScanState = COMPSERVER;
                        if (*(++argv[lArg]) == TEXT('\0'))
                        {
                            lArg++;
                        }
                        break;

					case TEXT('h'):				// Debug feature - tell first
						gfHeapCommand = TRUE;	// server to examine its heap
			            eScanState = DECOMPSERVER;
		                if (++lArg >= argc)
		                {
		                	return(0);
		                }
						break;

					case TEXT('f'):				// Debug feature - just try
						gfFindCommand = TRUE;	// to find servers
			            eScanState = DECOMPSERVER;
		                if (++lArg >= argc)
		                {
		                	return(0);
		                }
						break;

					default:
		                eScanState = ERR;
                        break;
                }

                if (lArg >= argc)
                {
                	return(0);
                }
                break;

			case DECOMPSERVER:
            	if (*(argv[lArg]) == TEXT('-'))
                {
                	argv[lArg]++;
                	eScanState = DASH;
                    break;
                }

            	if (lDecompSrvCnt < MAXDECOMPSERVERS)
                {
	            	gaszDecompServer[lDecompSrvCnt++] = argv[lArg];
                }
            	else if (lDecompSrvCnt == MAXDECOMPSERVERS)
                {
					Eprintf
					(
						TEXT("Warning: ignoring extra")
                        TEXT(" decompression server \"%s\"\n"),
						argv[lArg]
					);
				}

                if (++lArg >= argc)
                {
                	return(0);
                }
                break;

			case COMPSERVER:
            	if (*(argv[lArg]) == TEXT('-'))
                {
                	argv[lArg]++;
                	eScanState = DASH;
                    break;
                }

            	if (lCompSrvCnt < MAXCOMPSERVERS)
                {
	            	gaszCompServer[lCompSrvCnt++] = argv[lArg];
                }
                else if (lCompSrvCnt == MAXCOMPSERVERS)
                {
					Eprintf
					(
						TEXT("Warning: ignoring extra")
                        TEXT(" compression server \"%s\"\n"),
                        argv[lArg]
					);
				}

                if (++lArg >= argc)
                {
                	return(0);
                }
                break;

            default:
            	Eprintf(TEXT("ScanCommandLine() internal error\n"));
                exit(1);
        }
    }


    // Detected syntax error

    Eprintf
	(
		TEXT("Usage: %s <infile> [<outfile>] [<outformat>] ")
		TEXT("\n"), gacBaseName
	);
    return(1L);
}


//
//  FUNCTION: FindAndContactServers
//
//  PURPOSE: initiate communication with recompression servers via RPC Locator
//
//  PARAMETERS:
//
//  RETURN VALUE:
//    non-zero for error
//
//  COMMENTS:
//    Queries the name server for RPC bindings to servers.  Tell all servers
//    found to clear down their queues - this also tells us if they aren't
//    really there.
//

LONG		FindAndContactServers(void)
{
	RPC_STATUS RPC_ENTRY	status;
    RPC_NS_HANDLE			hImportContext;

	LPTSTR ProtSeq;
	LPTSTR NetworkAddr;
	LPTSTR EndPoint;
	LPTSTR ComputerName;
    LPTSTR pszStrBinding = NULL;

    unsigned long ulCode;

	gbaAction = RECOMPRESS;
	lDecompSrvCnt = 0L;

	Dprintf(TEXT("Contacting %scompression servers\n"),
						(gbaAction == RECOMPRESS ? TEXT("Re") : TEXT("De")));


	// Get Import Context for querying the name server

    RpcTryExcept {
        status = RpcNsBindingImportBegin(RPC_C_NS_SYNTAX_DEFAULT,
                                         NULL,
                                         DIST_COMP_C_IF_SPEC, 
                                         NULL,
                                         &hImportContext);
        Dprintf(TEXT("RpcNsBindingImportBegin returned 0x%x\n"), status);
    }
    RpcExcept(1) {
        ulCode = RpcExceptionCode();
        Dprintf(
        	TEXT("RPC Runtime raised exception 0x%x")
        	TEXT(" in RpcNsBindingImportBegin\n"),
        	ulCode);
		status = RPC_S_ENTRY_NOT_FOUND;
    }
    RpcEndExcept

    switch(status)
	{
		case RPC_S_OK:
			break;

		case RPC_S_ENTRY_NOT_FOUND:
	    	Eprintf(TEXT("Couldn't find any servers - terminating\n"));
			return(1L);

		default:
	   		Eprintf(TEXT("RpcNsBindingImportBegin failed, returned 0x%x\n"),
	    															status);
	        return(status);
	}


	// Loop until no more bindings are found

    for (;;)
    {
		// Get next binding from name server

        status = RpcNsBindingImportNext(hImportContext,
                                        &gahDecomp[lDecompSrvCnt]);
        Dprintf(TEXT("RpcNsBindingImportNext returned 0x%x\n"), status);

        if ( (status == RPC_S_NO_MORE_BINDINGS) ||
             (status == RPC_S_NAME_SERVICE_UNAVAILABLE ) )
            break;

        if (status != RPC_S_OK)
		{
        	Eprintf(TEXT("RpcNsBindingImportNext failed, returned 0x%x\n"),
        															status);
            return(status);
		}


		// Parse the binding 

        if (RpcBindingToStringBinding(gahDecomp[lDecompSrvCnt], &pszStrBinding)
        														!= RPC_S_OK)
		{
			Dprintf(TEXT("Invalid binding found\n"));
			continue;
		}

        Dprintf(TEXT("StringBinding: %s\n"), pszStrBinding );

		status = RpcStringBindingParse(pszStrBinding, NULL, &ProtSeq,
									&NetworkAddr, &EndPoint, NULL);
        if (status != RPC_S_OK)
		{
			Dprintf(TEXT("Invalid string binding \"%s\"\n"), pszStrBinding);
			continue;
		}


		// Skip over leading backslashes in network address

		for (ComputerName = NetworkAddr; *ComputerName == TEXT('\\');
																ComputerName++)
			;

		Dprintf(
			TEXT("ProtSeq: \"%s\"  ")
			TEXT("NetworkAddr: \"%s\"  ")
			TEXT("ComputerName: \"%s\"  ")
			TEXT("EndPoint: \"%s\"\n"),
			ProtSeq, NetworkAddr, ComputerName, EndPoint);

		RpcStringFree(&EndPoint);


		// Check whether this is a suitable protocol

		if
		(
				// Don't use non-LPC endpoints on the local machine

				(!_tcscmp(ComputerName, gszComputerName)
										&& _tcscmp(ProtSeq, TEXT("ncalrpc")))
			||

				// Don't try the LPC endpoint for other machines

				(_tcscmp(ComputerName, gszComputerName)
										&& !_tcscmp(ProtSeq, TEXT("ncalrpc")))
		)
		{
			Dprintf(
				TEXT("Ignoring protocol \"%s\" for computer \"%s\"\n"),
				ProtSeq, ComputerName);
			continue;
		}


		// Binding looks suitable, but it may be out of date - try it out by
		// instructing the server to clear itself down ready for a new stream.
		// If the call returns an error or causes an exception, consider this
		// server unusable

	    RpcTryExcept {
			status = RPCControl(gahDecomp[lDecompSrvCnt], RPCCTL_CLEARQUEUES);
	    }
	    RpcExcept(1) {
			status = !RPCCTL_OK;
	        ulCode = RpcExceptionCode();
	        Dprintf(
	        	TEXT("RPC Runtime raised exception 0x%x")
	        	TEXT(" in RPCControl contacting \"%s\"\n"),
	        	ulCode, ComputerName);
	    }
	    RpcEndExcept

		if (status == RPCCTL_OK)
		{
			Dprintf(
				TEXT("Successfully contacted server")
				TEXT(" via endpoint \"%s\" (%s)\n"),
				pszStrBinding,
				(gbaAction == RECOMPRESS ? TEXT("Recomp") : TEXT("Decomp")));
			
			gaszDecompServer[lDecompSrvCnt] = ComputerName;
			lDecompSrvCnt++;				
		}
		else
		{
			RpcStringFree(&NetworkAddr);
		}

		RpcStringFree(&ProtSeq);
		RpcStringFree(&pszStrBinding);
    }


	// Finish session with name server
	
	RpcTryExcept {
        status = RpcNsBindingImportDone(&hImportContext);
        Dprintf(TEXT("RpcNsBindingImportDone returned 0x%x\n"), status);
    }
    RpcExcept(1) {
        ulCode = RpcExceptionCode();
        Dprintf(
        	TEXT("RPC Runtime raised exception 0x%x")
        	TEXT(" in RpcNsBindingImportDone\n"),
        	ulCode);
    }
    RpcEndExcept


	if (lDecompSrvCnt == 0L)
	{
    	Eprintf(TEXT("Couldn't find any servers - terminating\n"));
		return(1L);
	}


	// Show the user what servers we found

   	Eprintf(TEXT("\n%scompression servers found:\n"),
   						(gbaAction == RECOMPRESS ? TEXT("Re") : TEXT("De")));
	for (lNextDecompSrv = 0; lNextDecompSrv < lDecompSrvCnt; lNextDecompSrv++)
    {
    	Eprintf(TEXT("    %s\n"), gaszDecompServer[lNextDecompSrv]);
    }

	Eprintf(TEXT("\n"));


	// In RECOMPRESS mode, compression and decompression servers are the same

	if (gbaAction == RECOMPRESS)
	{
		lCompSrvCnt = lDecompSrvCnt;

		for (lNextDecompSrv = 0;
							lNextDecompSrv < lDecompSrvCnt; lNextDecompSrv++)
	    {
			gahComp[lNextDecompSrv] = gahDecomp[lNextDecompSrv];
	    	gaszCompServer[lNextDecompSrv] = gaszDecompServer[lNextDecompSrv];
	    }
	}
                                                       
	return(gfFindCommand);		// If gfFindCommand is set, this
								// will prevent any actual processing
}


//
//  FUNCTION: ContactNamedServers
//
//  PURPOSE: initiate communication with recompression servers via RPC Locator
//
//  PARAMETERS:
//
//  RETURN VALUE:
//    non-zero for error
//
//  COMMENTS:
//    Tell all servers named on the command line to clear down their
//    queues - this also tells us if they aren't really there.
//

LONG		ContactNamedServers(void)
{
	RPC_STATUS RPC_ENTRY	status;
	TCHAR					*StringBinding;
	LONG					l;


	// Check user selections
	
	// If there are named servers and all are decompression servers (the
	// default setting if no option flag is used), use the RECOMPRESS setting
	// to force each bunch to be recompressed on a single server.
	
	// Otherwise use local server if none specified for either list

	gbaAction = DECOMPRESS;
	if ((lDecompSrvCnt != 0) && (lCompSrvCnt == 0))
	{
		gbaAction = RECOMPRESS;
	}
	else
	{
		if (lDecompSrvCnt == 0)
		{
			gaszDecompServer[0] = gszComputerName;
			lDecompSrvCnt = 1; 
		}
		
		if (lCompSrvCnt == 0)
		{
			gaszCompServer[0] = gszComputerName;
			lCompSrvCnt = 1; 
		}
	}

	if ((gbaAction == DECOMPRESS) && (lDecompSrvCnt > lCompSrvCnt))
	{
	   	Eprintf(
	   		TEXT("Error: can't have more decompression servers")
	   	 	TEXT("than compression servers\n"));
		return(1L);
	}


	// Tell the user what servers they asked for

   	Eprintf(TEXT("\n%scompression servers requested:\n"),
   						(gbaAction == RECOMPRESS ? TEXT("Re") : TEXT("De")));
	for (l = 0; l < lDecompSrvCnt; l++)
    {
    	Eprintf(TEXT("    %s\n"), gaszDecompServer[l]);
    }

    if (lCompSrvCnt)
    {
    	Eprintf(TEXT("\nCompression servers:\n"));
    	for (l = 0; l < lCompSrvCnt; l++)
        {
        	Eprintf(TEXT("    %s\n"), gaszCompServer[l]);
        }
    }

	Eprintf(TEXT("\n"));


	// Try to make contact with servers	- first the decompression servers

	Dprintf(TEXT("Contacting %scompression servers\n"),
						(gbaAction == RECOMPRESS ? TEXT("Re") : TEXT("De")));

	for (lNextDecompSrv = 0; lNextDecompSrv < lDecompSrvCnt; lNextDecompSrv++)
	{
		// Build string binding

		status = RpcStringBindingCompose
				 (
			 	   NULL,
				   (lstrcmpi(gaszDecompServer[lNextDecompSrv], gszComputerName)
								? TEXT("ncacn_nb_nb") : TEXT("ncalrpc")),
				   gaszDecompServer[lNextDecompSrv],
				   (lstrcmpi(gaszDecompServer[lNextDecompSrv], gszComputerName)
								? TEXT("127") : TEXT("DistComp")),
				   NULL,
				   &StringBinding
				 );
 
		if (status != RPC_S_OK)
		{
			Eprintf(TEXT("RpcStringBindingCompose() failed: error code %d\n"),
																	status);
			return(status);
		}

		Dprintf(TEXT("String binding is \"%s\"\n"), StringBinding);


		// Convert to a binding

		status = RpcBindingFromStringBinding(StringBinding,
												&gahDecomp[lNextDecompSrv]);
		if (status != RPC_S_OK)
		{
			Eprintf(
				TEXT("RpcBindingFromStringBinding() failed: error code %d\n"),
				status);
			return(status);
		}


		// Process the heap test debug option - tell the first named server to
		// scan its heap, then quit

		if (gfHeapCommand)
		{
		    RpcTryExcept
		    {
					RPCControl(gahDecomp[lNextDecompSrv], RPCCTL_HEAPSTATUS);
		    }
		    RpcExcept(1)
		    {
				status = RpcExceptionCode();
		        Eprintf
		        (
		        	TEXT("Exception %ld (0x%08.8lx) requesting")
		        	TEXT(" heap status from %scompression server \"%s\"\n"),
					status,
					status,
					(gbaAction == RECOMPRESS ? TEXT("re") : TEXT("de")),
					StringBinding
		        );
		    }
		    RpcEndExcept;

			return(1L);
		}


		// Test out communication with the server by instructing it to clear
		// itself down ready for a new stream.

	    RpcTryExcept
	    {
			if ((status = RPCControl(gahDecomp[lNextDecompSrv],
											RPCCTL_CLEARQUEUES)) == RPCCTL_OK)
			{
				Eprintf(
					TEXT("Successfully contacted server \"%s\" (%scomp)\n"),
					StringBinding,
					(gbaAction == RECOMPRESS ? TEXT("Re") : TEXT("De")));
			}
	    }
	    RpcExcept(1)
	    {
			status = RpcExceptionCode();
			if (status == RPC_S_SERVER_UNAVAILABLE)
			{
		        Eprintf
		        (
		        	TEXT("Couldn't contact %scompression server \"%s\"\n"),
					(gbaAction == RECOMPRESS ? TEXT("re") : TEXT("de")),
					StringBinding
		        );
			}
			else
			{
		        Eprintf
		        (
		        	TEXT("Exception %ld (0x%08.8lx)")
		        	TEXT(" contacting %scompression server \"%s\"\n"),
					status,
					status,
					(gbaAction == RECOMPRESS ? TEXT("re") : TEXT("de")),
					StringBinding
		        );
			}
	    }
	    RpcEndExcept;


		// In RECOMPRESS mode, compression and decompression servers are
		// the same

		if (gbaAction == RECOMPRESS)
		{
			gahComp[lNextDecompSrv] = gahDecomp[lNextDecompSrv];
			gaszCompServer[lNextDecompSrv] = gaszDecompServer[lNextDecompSrv];
		}
	}

	if (gbaAction == RECOMPRESS)
	{
		lCompSrvCnt = lDecompSrvCnt;
		return(0L);
	}


	// Now try to contact compression servers

	Dprintf(TEXT("Contacting compression servers\n"));
	for (lNextCompSrv = 0; lNextCompSrv < lCompSrvCnt; lNextCompSrv++)
	{
		status = RpcStringBindingCompose
				 (
				 	NULL,
					(lstrcmpi(gaszCompServer[lNextCompSrv], gszComputerName)
								? TEXT("ncacn_nb_nb") : TEXT("ncalrpc")),
					gaszCompServer[lNextCompSrv],
					(lstrcmpi(gaszCompServer[lNextCompSrv], gszComputerName)
								? TEXT("127") : TEXT("DistComp")),
					NULL, &StringBinding
				 );

		if (status != RPC_S_OK)
		{
			Eprintf(TEXT("RpcStringBindingCompose() failed: error code %d\n"),
																	status);
			return(status);
		}

		Dprintf(TEXT("String binding is \"%s\"\n"), StringBinding);

		status = RpcBindingFromStringBinding(StringBinding,
													&gahComp[lNextCompSrv]);
		if (status != RPC_S_OK)
		{
			Eprintf(
				TEXT("RpcBindingFromStringBinding() failed: error code %d\n"),
																	status);
			return(status);
		}

	    RpcTryExcept
	    {
			if ((status = RPCControl(gahComp[lNextCompSrv],
											RPCCTL_CLEARQUEUES)) == RPCCTL_OK)
			{
				Eprintf(TEXT("Successfully contacted server \"%s\" (Comp)\n"),
																StringBinding);
			}
	    }
	    RpcExcept(1)
	    {
			status = RpcExceptionCode();
			if (status == RPC_S_SERVER_UNAVAILABLE)
			{
		        Eprintf
		        (
		        	TEXT("Couldn't contact compression server \"%s\"\n"),
					StringBinding
		        );
			}
			else
			{
		        Eprintf
		        (
		        	TEXT("Exception %ld (0x%08.8lx)")
		        	TEXT(" contacting compression server \"%s\"\n"),
					status,
					status,
					StringBinding
		        );
			}
	    }
	    RpcEndExcept;
	}

	return(0L);
}


//
//  FUNCTION: _tmain (character-width-neutral version of main())
//
//  PURPOSE: entrypoint for the client application 
//
//  PARAMETERS:
//    argc - number of command line arguments
//    argv - array of command line arguments
//
//  RETURN VALUE:
//    End status - non-zero for error
//
//  COMMENTS:
//    Calls ScanCommandLine to interpret the command line.  If the user named
//    any server computers, calls ContactNamedServers to initialise the
//    recompression servers, otherwise calls FindAndContactServers to do this.
//    Opens the source and destination AVI files and processes the data using
//    ProcessBunches for video streams and CopyStream for non-video steams.
//

_tmain(int argc, TCHAR *argv[])
{
	LONG					lResult;
    int						j;
	TCHAR					acNameBuf[MAX_PATH];
	DWORD					dwLen;

	Eprintf(TEXT("DComp Distributed Compression Service demo client\n"));

	dwLen = MAX_COMPUTERNAME_LENGTH + 1;
	if (!GetComputerName(gszComputerName, &dwLen))
	{
		Eprintf(TEXT("Couldn't get computer name\n"));
		return(1);
	}

	if (gszComputerName != NULL)
	{
		Dprintf(TEXT("Computer name is \"%s\"\n"), gszComputerName);
	}

	gdwOSPlatformId = GetOSPlatformId();


	// Interpret the command line
	if (ScanCommandLine(argc, argv))
    {
    	return(1);
    }

	
	// Find output file name
	if (gszOutputFileName == NULL)
	{
		// No name given on command line: try to find a unique file name
        // for the output
		for (j = 0; j <= MAXNAMETRIES; j++)
		{
			_stprintf(acNameBuf, TEXT("vid%d.avi"), j);
			if ((_taccess(acNameBuf, 0) == -1) && (errno == ENOENT))
			{
				break;
			}
		}

		if (j > MAXNAMETRIES)
		{
			ErrExit(TEXT("Couldn't generate an output file name\n"));
            return(1);
		}

		gszOutputFileName = acNameBuf;
	}


	// Display details of this job

    if (gszInputFileName != NULL)
    {
    	Eprintf(TEXT("\nInput file:         %s\n"), gszInputFileName);
	    Eprintf(TEXT(  "Output file:        %s\n"), gszOutputFileName);
	    Eprintf(TEXT(  "Output compression: %4.4hs\n"), &gdwOutCompression);
    }


	// Connect with servers

	if
	(
			(gdwOSPlatformId == VER_PLATFORM_WIN32_NT)
		&&
			(((lDecompSrvCnt == 0L) && (lCompSrvCnt == 0L)) || gfFindCommand)
		&&
			!gfHeapCommand
	)
	{
		if (FindAndContactServers())
	    {
	    	return(1);
	    }
	}
	else
	{
		if (ContactNamedServers())
	    {
	    	return(1);
	    }
	}

	if ((lDecompSrvCnt == 0) || (lCompSrvCnt == 0))
	{
		Eprintf(TEXT("Server count was zero\n"));
		return(1);
	}


    AVIFileInit();

	// Open source AVI file
	Dprintf(TEXT("Opening \"%s\"\n"), gszInputFileName);
	if ((lResult = AVIFileOpen(&gpaviInFile, gszInputFileName, OF_READ, NULL))
                                                                      != 0L)
	{
		ErrExit(
            TEXT("Couldn't open source file \"%s\": error code %ld (0x%lx)\n"),
                                           gszInputFileName, lResult, lResult);
	}


	// Display file info for debugging
	DisplayFileInfo(gpaviInFile);


    // Create new AVI file
	_tremove(gszOutputFileName);
 	gpaviOutFile = NULL;
    if ((lResult = AVIFileOpen
    			   (
    			   		&gpaviOutFile,
    			   		gszOutputFileName,
    			   		OF_WRITE | OF_CREATE,
    			   		NULL
    			   )) != 0L)
 	{
		ErrExit(TEXT("Error code %ld (0x%lx) creating AVI file\n"),
															lResult, lResult);
	}

 	Dprintf(TEXT("Outputting to AVI file \"%s\"\n"), gszOutputFileName);


	// Open all input streams and initialise non-video output streams
	if ((lResult = InitStreams()) < 0L)
	{
		ErrExit(TEXT("Error opening streams in \"%s\"\n"), gszInputFileName);
	}

    // Display some stream debug info
	Dprintf(TEXT("Opened %ld streams in \"%s\"\n"), lResult, gszInputFileName); 
	DisplayStreamInfo();


	if ((lResult = AVIFileEndRecord(gpaviOutFile)) == 0)
	{
		ProcessBunches();
	}
	else
	{
		Eprintf(
			TEXT("Error code %ld(0x%lx)")
			TEXT(" writing end-of-record\n"),
			lResult, lResult);
	}


	// Close input and output streams streams
	for (--gcpavi; gcpavi >= 0; gcpavi--)
	{
		AVIStreamRelease(gapaviSrc[gcpavi]);
		AVIStreamRelease(gapaviDest[gcpavi]);
	}
 
	
	// Close the output file
	Dprintf(TEXT("Reference count of \"%s\" on closing was %ld\n"),
							gszOutputFileName, AVIFileRelease(gpaviOutFile));


	// Close input file and exit
	Dprintf(TEXT("Reference count of \"%s\" on closing was %ld\n"),
							gszInputFileName, AVIFileRelease(gpaviInFile));

	AVIFileExit();
	return(0);
}
